//----------------------------------------------------------------------------
//
// File:        bitmap.cpp
// Date:        06-Apr-2000
// Programmer:  Marc Rousseau
//
// Description:
//
// Copyright (c) 2000 Marc Rousseau, All Rights Reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
//
// Revision History:
//
//----------------------------------------------------------------------------

#include <memory.h>
#include <stdio.h>
#include "common.hpp"
#include "logger.hpp"
#include "tms9900.hpp"
#include "tms9918a.hpp"
#include "ti994a.hpp"
#include "SDL.h"
#include "bitmap.hpp"
#include "tms9918a-sdl.hpp"

cBitMap::cBitMap ( SDL_Surface *surface ) :
    m_Width ( 0 ),
    m_Height ( 0 ),
    m_Pitch ( 0 ),
    m_pSurface ( surface )
{
    FUNCTION_ENTRY ( this, "cBitMap ctor", true );

    if ( m_pSurface != NULL ) {
      m_Width  = m_pSurface->w;
      m_Height = m_pSurface->h;
      m_Pitch  = m_pSurface->pitch;
    }
}

cBitMap::~cBitMap ()
{
    FUNCTION_ENTRY ( this, "cBitMap dtor", true );
}

void cBitMap::SetPalette ( SDL_Color *palette, int count )
{
    FUNCTION_ENTRY ( this, "cBitMap::SetPalette", true );

    SDL_SetColors ( m_pSurface, palette, 0, count );
}

void cBitMap::LockSurface ()
{
    FUNCTION_ENTRY ( this, "cBitMap::LockSurface", true );

    SDL_LockSurface ( m_pSurface );
}

void cBitMap::UnlockSurface ()
{
    FUNCTION_ENTRY ( this, "cBitMap::UnlockSurface", true );

    SDL_UnlockSurface ( m_pSurface );
}

void cBitMap::Copy ( cBitMap *original )
{
    FUNCTION_ENTRY ( this, "cBitMap::Copy", true );

    LockSurface ();
    original->LockSurface ();

    UCHAR *pSrcData = original->GetData ();
    UCHAR *pDstData = GetData ();

    int size = m_Width * m_pSurface->format->BytesPerPixel;

    for ( int i = 0; i < m_Height; i++ ) {
        memcpy ( pDstData, pSrcData, size );
        pSrcData += original->Pitch ();
        pDstData += Pitch ();
    }

    original->UnlockSurface ();
    UnlockSurface ();
}
